%% -------- Bucher-Buorgund Response Surface Approach ---------------- %%
% 
% This script performs the quadratic response surface approach suggested
% by Bucher and Bourgund, and calculates the failure probability
% by FORM, Crude Monte Carlo and Importance Sampling. 
%
% Created by: David Larsson,    davidlastud.ntnu.no
%
% Part of the Master Thesis:
%
% "Methods of Reliability Analysis for Marine Structures"
%
%           Norwegian University of Science and Technology
%                 Department of Marine Technology
%                            June 2015    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


close all
clear all
clc

format long


%% Variable properties
mu1=20;                       % Plate thickness [mm]
mu2=100;                      % Axial Force [MPa]
mu3=0.1;                      % Lateral Pressure [MPa]
mu4=0;                        % Imperfection [mm]
mu5=315;                      % Yield Stress [MPa]


mu=[mu1 mu2 mu3 mu4 mu5];           % mean for R.Vs
dev= [0.02 0.2 0.1 0 0.07].*mu;     % std deviation of R.V
dev(4)=4; 
Acs=840*6*20+5*(420*10+140*16);     % Cross-section area in mm^2
nvar=length(mu);

fw=[2 1] ;                          % spread of sampling points

ptype=2;                            %    polynomial type
antype=1;                           %    analysis type 
                                    %    1: plate stress at midspan  
                                    %    2: ultimate collapse

%% ----------------- Opening result file ----------------------------- %%
file1=fopen('data_rs.txt','w+');
if file1 == -1
    error('Could not render output file')
else
    disp('data_rs.txt created')
end

if antype==1
fprintf(file1,'%s \r\n','Yield in plate at midspan');
elseif antype==2
    fprintf(file1,'%s \r\n','Linear buckling');
elseif antype==3
    fprintf(file1,'%s \r\n','Ultimate collapse');
else
    error('Invalid analysis type')
end

cent_pt=zeros(size(mu)); %1st center point at the mean
guess=cent_pt';          %1st guess for FORM at the means

for runs=1:2             %number of iterations

f=fw(runs);   % retrieving sample spread factor
 
%% -------------------- Generating sample ----------------------------- %% 
[samp_z,samp_x]=gen_samp(cent_pt,f,mu,dev);


samp_abaq=samp_x;   %sample in ABAQUS-units (N and m)

samp_abaq(:,1)=samp_abaq(:,1)./1000; %from mm to m
samp_abaq(:,2)=samp_abaq(:,2).*Acs;  %from MPa to N
samp_abaq(:,3)=samp_abaq(:,3).*1e6;  %from MPa to Pa
samp_abaq(:,4)=samp_abaq(:,4)./1000; %from mm to m
samp_abaq(:,5)=samp_abaq(:,5).*1e6;  %from MPa to Pa     

%% ---------------- Performing the sampling -------------------------- %%

out = abaqus_sample(samp_abaq, antype);

%% ---------------- Evaluating safety margin ------------------------ %%

if antype==1
    yield=samp_abaq(:,5);
    %safety margin normalized wrt yield stress
    gexp=(0.8*yield-out')./(mu5*1e6); 
    
elseif antype ==2
    
    % fixed axial load at mean in the ABAQUS-script,
    % so axial capacity is LPF(out) times mu2*Acs in N.
    ax_load=samp_abaq(:,2);
    gexp=(0.8*out'*mu2*Acs-ax_load)./(mu2*Acs);  % normalize wrt mean load
end


%% ---------- Printing the sampling points to file -------------------- %%
fprintf(file1,'%s %d \r\n','Experiment',runs);
for i=1:size(samp_x,1)
    for j=1:size(samp_x,2)
    fprintf(file1,'%7.5f \t',samp_x(i,j));
    end
fprintf(file1,'%7.3f \r\n',gexp(i));
end

%% -------- Fitting a quadratic polynomial without cross terms -------- %%

%x-space
mdl_x= fitlm(samp_x,gexp,'purequadratic');
par_x=mdl_x.Coefficients{:,1};

%z-space
mdl_z=fitlm(samp_z,gexp,'purequadratic');
param=mdl_z.Coefficients{:,1};


%% --------------------------- FORM ---------------------------------- %%

% remove old results
clear saf_ind despt grad

i=0;           % counter
diff=1;        % convergence operator
betatemp=100;  % beta dummy 

    % polynomial coefficients
    a=param(1);
    b=param(2:nvar+1);
    c=param(nvar+2:end);

    i=0;
    diff=1;
    betatemp=100;
while diff > 0.001
        i=i+1;
        dg=b+2*c.*guess;                        % derivatives
        alpha=-dg/sqrt(sum(dg.^2));             % sensitivity factors
        syms hlind
        hsolve=solve(hlind*sum(b.*alpha)...     % solve for beta
        +hlind^2*sum(c.*alpha.^2)+a==0,hlind);
    
        hs=eval(hsolve);
    %choosing the wanted solution
        if hs(1) < 0
            beta=hs(2);
        elseif hs(2) < 0
            beta=hs(1);
        elseif hs(2)>10
            beta=hs(1);
        elseif hs(1)>10
             beta=hs(2);
        else
            beta=max(hs);
        end
    
        dir_vec(i,:)=alpha;         % sensitivity factors
        guess=beta*alpha;           % guess update
        despt(i,:)=guess;           % store design point
        saf_ind(i)=beta;            % store beta
        diff=abs(beta-betatemp);    % convergence
        betatemp=beta;              % dummy update
        
    end


gradient=dg;                      % final gradients
dir=dir_vec(end,:);               % final sensitivity factors


despt_x=despt(end,:).*dev + mu;   % design point in original space
guess=despt(end,:)' ;             % guess for next loop

% probability of failure
pf_hl=normcdf(-saf_ind(end));

%% --------------- Sample the design point ---------------------------- %%

% abaqus-sample at design point
des_abaq=despt_x;

% transforming to abaqus-coordinates
des_abaq(:,1)=des_abaq(:,1)/1000;
des_abaq(:,2)=des_abaq(:,2)*Acs;
des_abaq(:,3)=des_abaq(:,3)*(1e6);
des_abaq(:,4)=des_abaq(:,4)/1000;
des_abaq(:,5)=des_abaq(:,5)*(1e6);

% sampling the design point
out2 = abaqus_sample(des_abaq, antype);

% evaluate safety margin at design point
 if antype ==1
    gexp2=(0.8*des_abaq(:,5)-out2)/(mu5*1e6);
 elseif antype ==2
    ax_load2=des_abaq(2);
    gexp2=(0.8*out2-ax_load2)./(mu2*Acs);  % normalize wrt mean ax load
                                              % from MPa to N;
 end

% Updating the center point for next sample set:
cent_pt_x = mu + gexp(1)*(despt_x-mu)./(gexp(1)-gexp2); %new center point

% z-coordinates
cent_pt=(cent_pt_x-mu)./dev;

if runs==2  % for the final response surface

%% ------- Crude MC with approximate limit state in z-domain ---------- %%

nfail=0;                    % no of failed samples
reps_mc=100;                % no of sample sets
nits_mc=1000000;            % no of samples in each set
nf_mc=zeros(1,reps_mc);     % failed samples history
nmc=zeros(1,reps_mc);       % no of samples history
for j=1:reps_mc
    disp('cmc')             % print progress trace
    disp(j)
    for i=1:nits_mc
        %generating sample pf variables 
        mcsamp=norminv(rand(1,nvar),0,1);
        % evaluate safety margin
            gsamp=param(1)+sum(param(2:nvar+1)'.*mcsamp)...
            +sum(param(nvar+2:end)'.*(mcsamp.^2)); 
        % check for failure
        if gsamp <= 0
        nfail=nfail+1;
        end

    end
        nf_mc(j)=nfail;
        nmc(j)=j*nits_mc;
end
pf_mc=nf_mc./nmc; % probability of failure history

%% -------------------- Importance sampling --------------------------- %%

reps_is=100;                        % no of sample sets
nis=zeros(1,reps_is);               % no of samples history
nfis=0;                             % no of failed samples counter
nits_is=1000;                       % no of samples in each set
pf_is=zeros(1,reps_is);             % probability of failure history

for j=1:reps_is
   hv=zeros(1,nits_is);             % IS-sample
   fx=zeros(1,nits_is);             % IS-function
   ind=zeros(1,nits_is);            % failure indicator
   g_is=zeros(1,nits_is);           % safety margin
   disp('IS')                       % progress trace
   disp(j)
   for i=1:nits_is
    vhat=mvnrnd(despt(end,:),eye(nvar));            % sample
    hv(i)=mvnpdf(vhat,despt(end,:),eye(nvar));      % IS function 
    fx(i)=mvnpdf(vhat);                             % joint pdf
    
    % evaluate safety margin
    g_is(i)=param(1)+sum(param(2:nvar+1)'.*vhat)...
            +sum(param(nvar+2:end)'.*(vhat.^2)); 
    if g_is(i) <=0
      ind(i)=1;
     else
      ind(i)=0;
     end
   end
   nfis=nfis+sum(ind.*fx./hv);
   nis(j)=j*nits_is;
   pf_is(j)=nfis/nis(j);
end


%% Printing to file
fprintf(file1,'\r\n%s \t','Center point(x-space):')
fprintf(file1,'%7.5f ',samp_x(1,:))
fprintf(file1,'\r\n%s \t','Design point (x-space):')
fprintf(file1,'%7.5f ',despt_x)
fprintf(file1,'\r\n%s \t','Design point (z-space):')
fprintf(file1,'%7.5f ',despt(end,:))
fprintf(file1,'\r\n%s %7.5f','Safety Margin at design point: ',gexp2)
fprintf(file1,'\r\n%s \t','Parameters in z-space:')
fprintf(file1,'%7.5f ',param)
fprintf(file1,'\r\n%s \t','Slope:')
fprintf(file1,'%7.5f ',gradient)
fprintf(file1,'\r\n%s \t','Sensitivity factors:')
fprintf(file1,'%7.5f ',alpha)
fprintf(file1,'\r\n%s %7.5f \r\n','Beta (FORM): ',saf_ind(end))
fprintf(file1,'%s %4e \r\n','Pf FORM:',pf_hl)
fprintf(file1,'%s %4e \r\n','Pf CMC:',pf_mc(end))
fprintf(file1,'%s %4e \r\n','No of samples, CMC:',nmc(end))
fprintf(file1,'%s %4e \r\n \r\n','Pf IS:',pf_is(end))
fprintf(file1,'%s %4e \r\n \r\n','No of samples, IS:',nis(end))
end


end

fclose(file1);