%% -------- Bucher-Buorgund Response Surface Approach ---------------- %%
% 
% This script performs the quadratic response surface approach suggested
% by Bucher and Bourgund, and calculates the failure probability
% by FORM, Crude Monte Carlo and Importance Sampling. 
%
% Created by: David Larsson,    davidlastud.ntnu.no
%
% Part of the Master Thesis:
%
% "Methods of Reliability Analysis for Marine Structures"
%
%           Norwegian University of Science and Technology
%                 Department of Marine Technology
%                            June 2015    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


close all
clear all
clc

format long


%% --------------------- Declaring input ------------------------------ %%

% Means of basic R.V's:
mu1=20;           % Plate thickness [mm]
mu2=100;          % Axial Force [MPa]
mu3=0.1;          % Lateral Pressure [MPa]
mu4=0;            % Imperfection [mm]
mu5=315;          % Yield Stress [MPa]


mu=[mu1 mu2 mu3 mu4 mu5];           % mean for r_v:s
dev= [0.02 0.2 0.1 0 0.07].*mu;     %std deviation of basic r.v
dev(4)=4; 

% parameters for weibull distributions
% corresponding to mean [100 0.1]
% std deviation [20 0.01];
scale=[107.9975 0.1043];
shape=[5.7974 12.1535];

% Parameters for log-normal yield stress
logmu=log((mu(5)^2)/sqrt(dev(5)+mu(5)^2));
logdev=sqrt(log(dev(5)^2/(mu(5)^2)+1));

nvar=length(mu);                    % no of variables

fw=[2 1] ;                          % factors for sample spread

% ptype=2;                            %    polynomial type
antype=1;                           %    analysis type 
                                    %    1: plate stress at midspan  
                                    %    2: axial capacity

%% ------------------ Opening result file ------------------------ %%
file1=fopen('data_rs.txt','w+');
if file1 == -1
    error('Could not render output file')
else
    disp('data_rs.txt created')
end

if antype==1
fprintf(file1,'%s \r\n','Yield in plate at midspan');
elseif antype==2
    fprintf(file1,'%s \r\n','Ultimate collapse');
else
    error('Invalid analysis type')
end

%% -------- Initiating Response-Surface calculations ---------------- %%

cent_pt=mu;      % 1st center point at the mean
guess_x=cent_pt; % 1st guess for FORM at the means

for runs=1:2     %number of iterations

f=fw(runs);   % retrieve sample spread factor
 
%% Generating sample points 
[samp_z,samp_x]=gen_samp_nonnorm(cent_pt,f,mu,dev);


samp_abaq=samp_x;   %sample in ABAQUS-units

Acs=840*6*20+5*(420*10+140*16);      %cs area in mm^2
samp_abaq(:,1)=samp_abaq(:,1)./1000; %from mm to M
samp_abaq(:,2)=samp_abaq(:,2).*Acs;  %from MPa to N
samp_abaq(:,3)=samp_abaq(:,3).*1e6;  %from MPa to Pa
samp_abaq(:,4)=samp_abaq(:,4)./1000; %from mm to m
samp_abaq(:,5)=samp_abaq(:,5).*1e6;  %from MPa to Pa     

%% ---------------- Performing the sampling -------------------------- %%

out = abaqus_sample(samp_abaq, antype);

%% ---------------- Evaluating safety margin ------------------------ %%
if antype==1
  
    yield=samp_abaq(:,5);
    %safety margin normalized wrt yield stress
    gexp=(0.8*yield-out')./(mu5*1e6); 
    
elseif antype ==2

% fixed axial load at mean in the ABAQUS-script,
% so axial capacity is LPF(out) times mu2*Acs in N.    
ax_load=samp_abaq(:,2);
gexp=(0.8*out*mu2*Acs-ax_load)./(mu2*Acs);  % normalize wrt mean ax load
                                    % from MPa to N;
    
end



%% ------------- Printing the sampling points to file ---------------- %%
fprintf(file1,'%s %d \r\n','Experiment',runs);
for i=1:size(samp_x,1)
    for j=1:size(samp_x,2)
    fprintf(file1,'%7.5f \t',samp_x(i,j));
    end
fprintf(file1,'%7.3f \r\n',gexp(i));
end



%% ------- Fitting a quadratic polynomial without cross terms ------- %%
mdl_x= fitlm(samp_x,gexp,'purequadratic');

par_x=mdl_x.Coefficients{:,1};


%% --------------------- FORM --------------------------------------- %%

% remove old results
clear saf_ind des_x grad

i=0;           % counter
diff=1;        % convergence operator
betatemp=100;  % beta dummy 

while diff > 0.001
i=i+1;
% approximate standard deviations
da1=dev(1);
da2=normpdf(norminv(wblcdf(guess_x(2),scale(1),shape(1))))...
    /wblpdf(guess_x(2),scale(1),shape(1));
da3=normpdf(norminv(wblcdf(guess_x(3),scale(2),shape(2))))...
    /wblpdf(guess_x(3),scale(2),shape(2));
da4=dev(4);
da5=normpdf(norminv(logncdf(guess_x(5),logmu,logdev)))...
    /lognpdf(guess_x(5),logmu,logdev);
da=[da1 da2 da3 da4 da5];

% approximate means
ma1=mu(1);
ma2=guess_x(2)-norminv(wblcdf(guess_x(2),scale(1),shape(1)))*da2;
ma3=guess_x(3)-norminv(wblcdf(guess_x(3),scale(2),shape(2)))*da3;
ma4=mu(4);
ma5=guess_x(5)-norminv(logncdf(guess_x(5),logmu,logdev))*da5;
ma=[ma1 ma2 ma3 ma4 ma5];


a=par_x(1);
b=par_x(2:nvar+1)';
c=par_x(nvar+2:end)';

% standardized variables
z=(guess_x-ma)./da;

% safety margin differentiated wrt standard gauss var
dg=b.*da + 2*c.*z.*(da.^2) + 2*c.*da.*ma;

% unit vectors
alpha=-dg/sqrt(sum(dg.^2));

syms hlind
hsolve = solve(a + sum(b.*(hlind*alpha.*da+ma))...
         + sum(c.*(hlind*alpha.*da + ma).^2) ==0,hlind);


hs=eval(hsolve);

%choosing the wanted solution
    if hs(1) < 0
       beta=hs(2);
    elseif hs(2) < 0
       beta=hs(1);   
    elseif hs(2)>10
       beta=hs(1);
    elseif hs(1)>10
       beta=hs(2);
    else
       beta=max(hs);
    end

% updating guess
guess_x=beta*alpha.*da+ma;

% Storing results
saf_ind(i)=beta;
des_x(i,:)=guess_x;
grad(i,:)=dg;
diff=abs(beta-betatemp);
betatemp=beta;
end



gradient=grad(end,:);  % storing gradient at deisgn point
despt_x=des_x(end,:);  % storing last design point
guess=despt_x;         % updating the guess for next round

% Probability of failure from FORM
pf_hl=normcdf(-saf_ind(end));

%% ------------------ Updating center point ------------------------- %%

% design point for pass to ABAQUS
des_abaq=despt_x;

% transforming to abaqus-units (N and m)
des_abaq(:,1)=des_abaq(:,1)/1000;
des_abaq(:,2)=des_abaq(:,2)*Acs;
des_abaq(:,3)=des_abaq(:,3)*(1e6);
des_abaq(:,4)=des_abaq(:,4)/1000;
des_abaq(:,5)=des_abaq(:,5)*(1e6);

% sampling the design point
out2 = abaqus_sample(des_abaq, antype);

% evaluate safety margin at design point
 if antype ==1
   % gexp2=(0.8*des_abaq(:,5)-out2)/des_abaq(:,5);
    gexp2=(0.8*des_abaq(:,5)-out2)/(mu5*1e6);
 elseif antype ==2
    ax_load2=des_abaq(2);
    gexp2=(0.8*out2*mu2*Acs-ax_load2)./(mu2*Acs);  
 end

% Updating the center point for next sample set:
cent_pt = mu + gexp(1)*(despt_x-mu)./(gexp(1)-gexp2); %new center point

if runs == 2

%% ----------- Crude Monte Carlo with response surface ------------- %%

nfail=0;                      % no of failed samples
reps_mc=100;                  % no of sample sets
nits_mc=1000000;              % no of samples/set
nf_mc=zeros(1,reps_mc);       % no of failed samples/set
nmc=zeros(1,reps_mc);         % no of samples totally so far
for j=1:reps_mc
    disp('crude mc')      
    disp(j)                   % print screen for  progress trace
    for i=1:nits_mc
        
        % drawing the sample
        mc1=norminv(rand,mu(1),dev(1));
        mc2=wblinv(rand,scale(1),shape(1));
        mc3=wblinv(rand,scale(2),shape(2));
        mc4=norminv(rand,mu(4),dev(4));
        mc5=logninv(rand,logmu,logdev);
        mcsamp=[mc1 mc2 mc3 mc4 mc5];
        
        % evaluating safety margin
            gsamp=par_x(1)+sum(par_x(2:nvar+1)'.*mcsamp)...
            +sum(par_x(nvar+2:end)'.*(mcsamp.^2)); 
        if gsamp <= 0
        nfail=nfail+1;
        end

    end
        nf_mc(j)=nfail;
        nmc(j)=j*nits_mc;
end
pf_mc=nf_mc./nmc;    % finding the probability of failure history


%% ---------- Importance Sampling from response surface ------------- %%

reps_is=100;                   % no of sample sets                     
nis=zeros(1,reps_is);          % no of samples so far
nfis=0;                        % failed samples operator
nits_is=1000;                  % no of samples in each set
pf_is=zeros(1,reps_is);        % probability of failure history

for j=1:reps_is
   hv=zeros(1,nits_is);         % IS-function values     
   fx=zeros(1,nits_is);         % random variable pdf values
   ind=zeros(1,nits_is);        % failure indicator
   g_is=zeros(1,nits_is);       % safety margin
    disp('IS')
    disp(j)                     % print screeen for  progress trace
   
   for i=1:nits_is
    % drawing from IS-function   
    vhat=mvnrnd(despt_x,diag(da.^2));
    
    %corresponding values of IS-function
    hv(i)=mvnpdf(vhat,despt_x,diag(da.^2));
    
    %values of variable PDF's
    fx1=normpdf(vhat(1),mu(1),dev(1));
    fx2=wblpdf(vhat(2),scale(1),shape(1));
    fx3=wblpdf(vhat(3),scale(2),shape(2));
    fx4=normpdf(vhat(4),mu(4),dev(4));
    fx5=lognpdf(vhat(5),logmu,logdev);
    
    %variable Joint PDF (independent variables)
    fx(i)=fx1*fx2*fx3*fx4*fx5;
    
    % evaluate safety amrgin
    g_is(i)=par_x(1)+sum(par_x(2:nvar+1)'.*vhat)...
            +sum(par_x(nvar+2:end)'.*(vhat.^2)); 
    if g_is(i) <=0
      ind(i)=1;
     else
      ind(i)=0;
     end
   end
   nfis=nfis+sum(ind.*fx./hv);
   nis(j)=j*nits_is;
   pf_is(j)=nfis/nis(j);
end


%% ----------------- Printing to file ------------------------------- %%
fprintf(file1,'\r\n%s \t','Center point:')
fprintf(file1,'%7.5f ',samp_x(1,:))
fprintf(file1,'\r\n%s \t','Design point:')
fprintf(file1,'%7.5f ',despt_x)
fprintf(file1,'\r\n%s %7.5f','Safety Margin at design point: ',gexp2)
fprintf(file1,'\r\n%s \t','Parameters:')
fprintf(file1,'%7.5f ',par_x)
fprintf(file1,'\r\n%s \t','Slope:')
fprintf(file1,'%7.5f ',gradient)
fprintf(file1,'\r\n%s \t','Sensitivity factors:')
fprintf(file1,'%7.5f ',alpha)
fprintf(file1,'\r\n%s %7.5f \r\n','Beta (FORM): ',saf_ind(end))
fprintf(file1,'%s %4e \r\n','Pf FORM:',pf_hl)
fprintf(file1,'%s %4e \r\n','Pf CMC:',pf_mc(end))
fprintf(file1,'%s %4e \r\n','No of samples, CMC:',nmc(end))
fprintf(file1,'%s %4e \r\n \r\n','Pf IS:',pf_is(end))
fprintf(file1,'%s %4e \r\n \r\n','No of samples, IS:',nis(end))
end

end
fclose(file1);