%%%%%%%%%%%%%% Bucher-Bourgund Response Surface Approach %%%%%%%%%%%%%%
% 
% This script performs the response surface approach suggested
% by Bucher and Bourgund, and calculates the failure probability
% by different methods.
%
% Created by: David Larsson,        davidla@stud.ntnu.no
%
% Part of the Master's Thesis:
%
% "Methods of Reliability Analysis for Marine Structures"
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


close all
clear all
clc
format long

%% --------------------- Variable properties -------------------------- %%
mu1=20;           % Plate thickness [mm]
mu2=100;          % Axial Force [MPa]
mu3=0.1;          % Lateral Pressure [MPa]
mu4=315;          % Yield Stress [MPa]


mu=[mu1 mu2 mu3 mu4];           % mean for r_v:s
dev= [0.02 0.2 0.1 0.07].*mu;  %std deviation of basic r.v

Acs=840*6*20+5*(420*10+140*16); % mean cs area in mm^2

nvar=length(mu);

fw=[3 3] ;                          % factor for spread of sampling

antype=4;                           %    analysis type 
                                    %    1: plate stress at midspan  
                                    %    2: linear buckling
                                    %    3: ultimate collapse

%% --------- Opening result file and set first sample center ---------- %%
file1=fopen('data_rs.txt','w+');
fprintf(file1,'%s \r\n','DNV-RPC201');

cent_pt=zeros(1,nvar); %1st center point at the mean
guess=cent_pt';        %1st guess for FORM at the mean


%% ---------------- Initiate response surface loop -------------------- %%
for runs=1:2   %number of iterations

%% -------------------- Create sample --------------------------------- %%   
% find sample spread    
f=fw(runs);   % f=2 for first round, f=1 for 2nd round

% draw sample
[samp_z,samp_x]=gen_samp(cent_pt,f,mu,dev);


%% ------------------ Perform sampling -------------------------------- %%

    for i=1:size(samp_x,1)
        checks=dnvcheck(samp_x(i,1),samp_x(i,2),samp_x(i,3),samp_x(i,4));
        gexp(i)=0.8-checks(4);
    end
    
       

%% ----------- Printing the sampling points to file ------------------- %%
fprintf(file1,'%s %d \r\n','Experiment',runs);
for i=1:size(samp_x,1)
    for j=1:size(samp_x,2)
    fprintf(file1,'%7.5f \t',samp_x(i,j));
    end
fprintf(file1,'%7.3f \r\n',gexp(i));
end

%% ------ Fitting quadratic polynomial without cross terms ------------ %%
mdl = fitlm(samp_z,gexp,'purequadratic');
param=mdl.Coefficients{:,1};

%% --------------------------- FORM ----------------------------------- %%

%remove old results
clear safind despt grad

% polynomial coefficients
a=param(1);
b=param(2:nvar+1);
c=param(nvar+2:end);

i=0;            % itereation counter
diff=1;         % convergence parameter
betatemp=100;   % safety index dummy
% initiate FORM-loop
while diff > 0.001
        i=i+1; 
        dg=b+2*c.*guess;            % partial derivatives
        alpha=-dg/sqrt(sum(dg.^2)); % unit vectors
        
        % solve for beta
        syms hlind
        hsolve=solve(hlind*sum(b.*alpha)...
        +hlind^2*sum(c.*alpha.^2)+a==0,hlind);
        hs=eval(hsolve);

    %choosing the wanted solution
        if hs(1) < 0
            beta=hs(2);
        elseif hs(2) < 0
            beta=hs(1);
        elseif hs(2)>10
            beta=hs(1);
        elseif hs(1)>10
             beta=hs(2);
        else
            beta=max(hs);
        end
        
        % store data
        dir_vec(i,:)=alpha; 
        guess=beta*alpha;
        despt(i,:)=guess;
        saf_ind(i)=beta;
        
        % update convergence
        diff=abs(beta-betatemp);
        betatemp=beta;
        
    end


gradient=dg;                      % retrieving the partial derivatives
dir=dir_vec(end,:);               % last unit vector set

despt_x=despt(end,:).*dev + mu;   % design point in original space
guess=despt(end,:)' ;             % update guess


 
%% ----------- Updating center point for new sample ------------------- %%

gexp2=0.8-dnvcheck(despt_x(1),despt_x(2),despt_x(3),despt_x(4));
gexp2=gexp2(4);

cent_x = mu + gexp(1)*(despt_x-mu)./(gexp(1)-gexp2); %new center point
cent_pt=(cent_x-mu)./dev; %standardizing the center point


%% -------------- Simulations and Postprocessing ---------------------- %%

if runs==2

%% -------------- Probability of failure from FORM -------------------- %%

% failure probability FORM
pf_hl=normcdf(-saf_ind(end));

%% ----------- Crude MC with approximate limit state ------------------ %%

nfail=0;                    % no of failed samples
reps_mc=100;                % no of sample sets
nits_mc=1000;            % no of samples in each set
nf_mc=zeros(1,reps_mc);     % no of failed samples in each set
nmc=zeros(1,reps_mc);       % no of samples counter
for j=1:reps_mc
    disp('crudemc')    % printscreen for monitoring
    disp(j)
    for i=1:nits_mc
        
        % draw sample
        mcsamp=norminv(rand(1,nvar),0,1);
        % evaluate safety margin
        gsamp=param(1)+sum(param(2:nvar+1)'.*mcsamp)...
        +sum(param(nvar+2:end)'.*(mcsamp.^2)); 
        if gsamp <= 0
        nfail=nfail+1;
        end

    end
        nf_mc(j)=nfail;
        nmc(j)=j*nits_mc;
end
pf_mc=nf_mc./nmc;

%% ------------------- Importance Sampling ---------------------------- %%

reps_is=100;                % no of sample sets
nis=zeros(1,reps_is);       % no of samples counter
nfis=0;                     % no of failed samples
nits_is=1000;               % no of samples in each set
pf_is=zeros(1,reps_is);     % probability of failure estimator

for j=1:reps_is
    disp('impsamp')         % print screen for monitoring
    disp(j)
   hv=zeros(1,nits_is);     % importance sampling function
   fx=zeros(1,nits_is);     % joint pdf of R.V
   ind=zeros(1,nits_is);    % failure indicator
   g_is=zeros(1,nits_is);   % safety margin
   
   for i=1:nits_is
    % drawing importance sample   
    vhat=mvnrnd(despt(end,:),eye(nvar));
    % evaluate I.S function
    hv(i)=mvnpdf(vhat,despt(end,:),eye(nvar));
    % evaluate joint pdf
    fx(i)=mvnpdf(vhat);
    % evaluate safety margin
    g_is(i)=param(1)+sum(param(2:nvar+1)'.*vhat)...
            +sum(param(nvar+2:end)'.*(vhat.^2)); 
    if g_is(i) <=0
      ind(i)=1;
     else
      ind(i)=0;
     end
   end
   nfis=nfis+sum(ind.*fx./hv);
   nis(j)=j*nits_is;
   pf_is(j)=nfis/nis(j);
end

%% Printing to file
fprintf(file1,'\r\n%s \t','Center point:')
fprintf(file1,'%7.5f ',samp_x(1,:))
fprintf(file1,'\r\n%s \t','Design point:')
fprintf(file1,'%7.5f ',despt_x)
fprintf(file1,'\r\n%s %7.5f','Safety Margin at design point: ',gexp2)
fprintf(file1,'\r\n%s \t','Parameters:')
fprintf(file1,'%7.5f ',param)
fprintf(file1,'\r\n%s \t','Slope:')
fprintf(file1,'%7.5f ',gradient)
fprintf(file1,'\r\n%s \t','Sensitivity factors:')
fprintf(file1,'%7.5f ',alpha)
fprintf(file1,'\r\n%s %7.5f \r\n','Beta (FORM): ',saf_ind(end))
fprintf(file1,'%s %4e \r\n','Pf FORM:',pf_hl)
fprintf(file1,'%s %4e \r\n','Pf CMC:',pf_mc(end))
fprintf(file1,'%s %4e \r\n','No of samples, CMC:',nmc(end))
fprintf(file1,'%s %4e \r\n \r\n','Pf IS:',pf_is(end))
fprintf(file1,'%s %4e \r\n \r\n','No of samples, IS:',nis(end))
end


end
fclose(file1);

