%%%%%%%%%%%%%% Bucher-Bourgund Response Surface Approach %%%%%%%%%%%%%%
% 
% This script performs the response surface approach suggested
% by Bucher and Bourgund, and calculates the failure probability
% by different methods for each iteration. 
%
% Created by: David Larsson, davidla@stud.ntnu.no
%
% Part of the Master's Thesis:
%
% "Methods of Reliability Analysis for Marine Structures"
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


close all
clear all
clc

format long


%% Variable properties
mu1=20;           % Plate thickness [mm]
mu2=100;          % Axial Force [MPa]
mu3=0.1;          % Lateral Pressure [MPa]
mu5=315;          % Yield Stress [MPa]
Acs=840*6*20+5*(420*10+140*16); %cs area in mm^2

mu=[mu1 mu2 mu3 mu5];           % mean for r_v:s
dev= [0.02 0.2 0.1 0.07].*mu;  %std deviation of basic r.v

%parameters for weibull distributions
% corresponding to mean [100 0.1]
% std deviation [20 0.01];
scale=[107.9975 0.1043];
shape=[5.7974 12.1535];

%parameters for log-normal yield stress
logmu=log((mu(4)^2)/sqrt(dev(4)+mu(4)^2));
logdev=sqrt(log(dev(4)^2/(mu(4)^2)+1));

nvar=length(mu);

fw=[2 1] ;                          % factor for spread of sampling points

ptype=2;                            % polynomial type
antype=4;                           %    analysis type 
                                    %    1: plate stress at midspan  
                                    %    2: linear buckling
                                    %    3: ultimate collapse
                                    %    4: DNV RP-C201 plate midspan

check=0;

%% Opening result file
file1=fopen('data_rs.txt','w+');
if file1 == -1
    error('Could not render output file')
else
    disp('data_rs.txt created')
end

fprintf(file1,'%s \r\n','DNV-RPC201');


cent_pt=mu; %1st center point at the mean
guess_x=cent_pt; %1st guess for FORM at the mean

for runs=1:2   %number of iterations

 f=fw(runs);   % f=2 for first round, f=1 for 2nd round
    
    
    
[samp_z,samp_x]=gen_samp_nonnorm(cent_pt,f,mu,dev);




%% Performing the sampling

    for i=1:size(samp_x,1)
        checks=dnvcheck(samp_x(i,1),samp_x(i,2),samp_x(i,3),samp_x(i,4));
        gexp(i)=0.8-checks(4);
    end
  
                                      

%% Printing the sampling points to file
fprintf(file1,'%s %d \r\n','Experiment',runs);
for i=1:size(samp_x,1)
    for j=1:size(samp_x,2)
    fprintf(file1,'%7.5f \t',samp_x(i,j));
    end
fprintf(file1,'%7.3f \r\n',gexp(i));
end



%% Fitting a quadratic polynomial without cross terms:
mdl_x= fitlm(samp_x,gexp,'purequadratic');

par_x=mdl_x.Coefficients{:,1};


%% --------------------------- FORM ----------------------------------- %%

% remove old results
clear saf_ind des_x grad

i=0;
diff=1;
betatemp=100;


while diff > 0.001
i=i+1;    
% approximate standard deviations
da1=dev(1);
da2=normpdf(norminv(wblcdf(guess_x(2),scale(1),shape(1))))...
    /wblpdf(guess_x(2),scale(1),shape(1));
da3=normpdf(norminv(wblcdf(guess_x(3),scale(2),shape(2))))...
    /wblpdf(guess_x(3),scale(2),shape(2));
da4=normpdf(norminv(logncdf(guess_x(4),logmu,logdev)))...
    /lognpdf(guess_x(4),logmu,logdev);
da=[da1 da2 da3 da4];

% approximate means
ma1=mu(1);
ma2=guess_x(2)-norminv(wblcdf(guess_x(2),scale(1),shape(1)))*da2;
ma3=guess_x(3)-norminv(wblcdf(guess_x(3),scale(2),shape(2)))*da3;
ma4=guess_x(4)-norminv(logncdf(guess_x(4),logmu,logdev))*da4;
ma=[ma1 ma2 ma3 ma4];


a=par_x(1);
b=par_x(2:nvar+1)';
c=par_x(nvar+2:end)';


% standardized variables
z=(guess_x-ma)./da;

% safety margin differentiated wrt standard gauss var
dg=b.*da + 2*c.*z.*(da.^2) + 2*c.*da.*ma;

% unit vectors
alpha=-dg/sqrt(sum(dg.^2));

syms hlind
hsolve = solve(a + sum(b.*(hlind*alpha.*da+ma))...
         + sum(c.*(hlind*alpha.*da + ma).^2) ==0,hlind);


hs=eval(hsolve);

    %choosing the wanted solution
        if hs(1) < 0
            beta=hs(2);
        elseif hs(2) < 0
            beta=hs(1);
            

        elseif hs(2)>10
            beta=hs(1);
        elseif hs(1)>10
             beta=hs(2);
       else
            beta=max(hs);
        end

% updating guess
guess_x=beta*alpha.*da+ma;

saf_ind(i)=beta;
des_x(i,:)=guess_x;
grad(i,:)=dg;
diff=abs(beta-betatemp);
betatemp=beta;
end

despt_x=des_x(end,:);
gradient=grad(end,:);

% updating the guess for next form-loop
guess=despt_x;

%% ------------ Updating center point for new sample ----------------- %%

gexp2=0.8-dnvcheck(despt_x(1),despt_x(2),despt_x(3),despt_x(4));
gexp2=gexp2(4);
cent_pt = mu + gexp(1)*(despt_x-mu)./(gexp(1)-gexp2); %new center point

 

if runs==2

%% ---------- Probability of failure from FORM ------------------------ %%

pf_hl=normcdf(-saf_ind(end));

%% ---------- Crude Monte Carlo with response surface ----------------- %%


nfail=0;                 % no of failed samples
reps_mc=100;             % no of sample sets
nits_mc=1000;         % no of its in each set
nf_mc=zeros(1,reps_mc);  % no of failed samples
nmc=zeros(1,reps_mc);    % no of samples counter
for j=1:reps_mc
    disp('crude mc')     % print screen for monitoring
    disp(j)
    for i=1:nits_mc
        % sampling
        mc1=norminv(rand,mu(1),dev(1));
        mc2=wblinv(rand,scale(1),shape(1));
        mc3=wblinv(rand,scale(2),shape(2));
        mc4=logninv(rand,logmu,logdev);
        mcsamp=[mc1 mc2 mc3 mc4];
        % evaluate safety margin
        gsamp=par_x(1)+sum(par_x(2:nvar+1)'.*mcsamp)...
        +sum(par_x(nvar+2:end)'.*(mcsamp.^2)); 
        if gsamp <= 0
            nfail=nfail+1;
        end

    end
        nf_mc(j)=nfail;
        nmc(j)=j*nits_mc;
end

% probability of failure estimator
pf_mc=nf_mc./nmc;


%% ----------- Importance Sampling from response surface -------------- %%

reps_is=100;                % no of sample sets
nis=zeros(1,reps_is);       % no of samples counter
nfis=0;                     % no of failed samples
nits_is=1000;               % no of samples in each set
pf_is=zeros(1,reps_is);

for j=1:reps_is
   hv=zeros(1,nits_is);     % importance sampling function
   fx=zeros(1,nits_is);     % basic R.V  joint pdf
   ind=zeros(1,nits_is);    % failure indicator
   g_is=zeros(1,nits_is);   % safety margin
    disp('IS')
    disp(j)
   for i=1:nits_is
    % drawing importance sample variable   
    vhat=mvnrnd(despt_x,diag(da.^2));
    % evaluate improtance sampling function
    hv(i)=mvnpdf(vhat,despt_x,diag(da.^2));
    % evaluate joint pdf of R.V
    fx1=normpdf(vhat(1),mu(1),dev(1));
    fx2=wblpdf(vhat(2),scale(1),shape(1));
    fx3=wblpdf(vhat(3),scale(2),shape(2));
    fx4=lognpdf(vhat(4),logmu,logdev);
    fx(i)=fx1*fx2*fx3*fx4;
    % evaluate safety margin
    g_is(i)=par_x(1)+sum(par_x(2:nvar+1)'.*vhat)...
            +sum(par_x(nvar+2:end)'.*(vhat.^2)); 
    if g_is(i) <=0
      ind(i)=1;
     else
      ind(i)=0;
     end
   end
   nfis=nfis+sum(ind.*fx./hv);
   nis(j)=j*nits_is;
   pf_is(j)=nfis/nis(j);
end


%% Printing to file
fprintf(file1,'\r\n%s \t','Center point:')
fprintf(file1,'%7.5f ',samp_x(1,:))
fprintf(file1,'\r\n%s \t','Design point:')
fprintf(file1,'%7.5f ',despt_x)
fprintf(file1,'\r\n%s %7.5f','Safety Margin at design point: ',gexp2)
fprintf(file1,'\r\n%s \t','Parameters:')
fprintf(file1,'%7.5f ',par_x)
fprintf(file1,'\r\n%s \t','Slope:')
fprintf(file1,'%7.5f ',gradient)
fprintf(file1,'\r\n%s \t','Sensitivity factors:')
fprintf(file1,'%7.5f ',alpha)
fprintf(file1,'\r\n%s %7.5f \r\n','Beta (FORM): ',saf_ind(end))
fprintf(file1,'%s %4e \r\n','Pf FORM:',pf_hl)
fprintf(file1,'%s %4e \r\n','Pf CMC:',pf_mc(end))
fprintf(file1,'%s %4e \r\n','No of samples, CMC:',nmc(end))
fprintf(file1,'%s %4e \r\n \r\n','Pf IS:',pf_is(end))
fprintf(file1,'%s %4e \r\n \r\n','No of samples, IS:',nis(end))
end

end

fclose(file1);