function [checks] = dnvcheck(t,S_xsd,p_sd,f_y)

%% ----------------------- dnvcheck.m ------------------------------ %% 
%
% PURPOSE: Check the DNV-RP-C201 criteria for T-bar stiffened panel
%          subject to lateral pressure at plate side and compressive 
%          axial stress
%
% INPUT: t          - Plate thickness [mm]
%        S_xsd      - Axial stress (positive in compression) [MPa]
%        p_sd       - Design lateral pressure (acting on plate side) [MPa]
%        f_y        - Yield Stress [MPa]
%
%
% Author: David Larsson,        davidla@stud.ntnu.no
%
% June 2015
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% close all
% clear all
% clc


%% ------------------------- Fixed Input ------------------------------ %%

% t=20;
% S_xsd=100;
% p_sd=0.1;
% f_y=315;


E=206000;               % elastic modulus [MPa]
l=4000;                 % panel length (stiffener direction) [m]
s=840;                  % distance between stiffeners
t_web=10;               % stiffener web thickness [mm]
h_web=420;              % stiffener web height [mm]
t_fl=16;                % stiffener flange thickness [mm]
w_fl=140;               % stiffener flange width     [mm]
gamma_m=1.15 ;          % material factor
poiss=0.3;              % poissons number
G=E/(2+2*poiss);        % shear modulus;

zstar=0;                % working point of axial load from N.A
u=0;                    % tension field action

%% equivalent lateral line load:

p_0=0;                      % zero transverse compression
q_sd=(p_sd+p_0)*s ;         


%% reduced slenderness
lambar_p=0.525*s*sqrt(f_y/E)/t;

if lambar_p > 0.673
C_xs=(lambar_p-0.22)/lambar_p^2;
else
    C_xs=1;
end

C_ys=1; % reduction factor due to transverse stress

s_e=s*C_xs*C_ys; % Effective plate width

%% Effective area
A_s=t_web*h_web+t_fl*w_fl; % stiffener c.s area
A_e=(A_s + s_e*t);         % area of efficient c.s

% Effective 2nd mom of area
[z_p,z_t,I_e] = smomarea(s_e,w_fl,t,t_fl,t_web,h_web);

%elastic section modulus
W_ep=I_e/z_p;   %plate side
W_es=I_e/z_t;   %stiff side
if W_ep <= W_es
    W=W_ep;
else
    W=W_es;
end
p_f=12*W*f_y/(l*l*s*gamma_m);
l_k=l*(1-0.5*abs(p_sd/p_f)); % buckling length

i_e=sqrt(I_e/A_e);           % radius of gyration


f_e=pi*pi*E*(i_e/l_k)^2;       % euler buckling strength


%% torsional buckling
A_f=t_fl*w_fl;               % Flange cross-section area
A_w=t_web*h_web;             % Web cross-section area


I_z=A_f*w_fl*w_fl/12;        % Stiffener S.M.A




%% Resistance parameters for stiffeners

% yield resistance
N_rd=A_e*f_y/gamma_m; 

% stiffener-induced axial buckling resistance
N_ksrd=A_e*fkcalc(f_y*frss(t_web,h_web,...
       t_fl,w_fl,E,I_z,f_y,l)...
       ,f_e,i_e,z_t)/gamma_m;                  % use frss equation

   
% plate-induced axial buckling resistance   
N_kprd=A_e*fkcalc(f_y,f_e,i_e,z_p)/gamma_m;    % use f_r = f_y


M_s1rd=W_es*frss(t_web,h_web,t_fl,w_fl,E,I_z,f_y,0.4*l)/gamma_m;
M_s2rd=W_es*frss(t_web,h_web,t_fl,w_fl,E,I_z,f_y,0.8*l)/gamma_m;
 
M_strd=W_es*f_y/gamma_m;
M_prd=W_ep*f_y/gamma_m;

N_sd=S_xsd*(A_s+s*t);     % equivalent axial force


%% largest field moments
M_1sd=abs(q_sd*l*l/12);
M_2sd=M_1sd/2;

N_e=pi*pi*E*A_e/((l_k/i_e)^2); % euler buckling

%% interaction equations

f1=N_sd/N_ksrd+(M_1sd-N_sd*zstar)/(M_s1rd*(1-N_sd/N_e)) + u;

f2=N_sd/N_kprd-2*N_sd/N_rd+(M_1sd-N_sd*zstar)/(M_prd*(1-N_sd/N_e))+u;

f3=N_sd/N_ksrd-2*N_sd/N_rd+(M_2sd+N_sd*zstar)/(M_strd*(1-N_sd/N_e));

f4=N_sd/N_kprd+(M_2sd+N_sd*zstar)/(M_prd*(1-N_sd/N_e))+u;

checks=[f1 f2 f3 f4];


end