# ********************** scriptuls.py ************************ 	
#							  		
# Created for ABAQUS-scripting interface                
# Part of the Master Thesis:          	 	
# "Methods of Reliability Analysis for Marine Structures" 
#
# Norwegian University of Science and Technology      	
# Department of Marine Technology              	
# 								
# Purpose:
# Perform RIKS analysis of a Stiffened Panel subject                        	
# to axial compression and lateral pressure.                                          	  
#                                                        	
# Author: David Larsson, davidla@stud.ntnu.no             	
#                                                        	
# 								
#								
# Date:   June 2015						
#								
# *************************************************************




# FUNCTIONALITY

from abaqus import *
from abaqusConstants import *
from part import *
from material import *
from section import *
from assembly import *
from step import *
from interaction import *
from load import *
from mesh import *
from optimization import *
from job import *
from sketch import *
from visualization import *
from connectorBehavior import *
from math import *
from numpy import *

backwardCompatibility.setValues(includeDeprecated=True,
                                reportDeprecated=True)

#LOAD VARIABLES
execfile('Var.py')
ElMod = 206000000000


# CREATE THE MODEL
#Create Model
myMod = mdb.Model(name='StiffPanel')
execfile('genmod.py')

# INITIAL STEP

# Impose BC on left edge (through reference point constraints)
myMod.DisplacementBC(amplitude=UNSET, createStepName='Initial', 
    distributionType=UNIFORM, fieldName='', localCsys=None, name='BC_left', 
    region=Region(referencePoints=(
    myAssembly.referencePoints[leftRP.id], )), u1=SET, u2=UNSET, 
    u3=UNSET, ur1=UNSET, ur2=SET, ur3=SET)
    
# Impose BC on right edge (through reference point constraints)    
myMod.DisplacementBC(amplitude=UNSET, createStepName='Initial', 
    distributionType=UNIFORM, fieldName='', localCsys=None, name='BC-2', 
    region=Region(referencePoints=(
    myAssembly.referencePoints[rightRP.id], )), u1=SET, u2=SET, 
    u3=SET, ur1=UNSET, ur2=UNSET, ur3=UNSET)
    
    
# Impose BC on longitudinal edges    
myMod.DisplacementBC(amplitude=UNSET, createStepName='Initial', 
    distributionType=UNIFORM, fieldName='', localCsys=None, name='BC-3', 
    region=myAssembly.sets['longedges'], u1=SET, u2=SET, 
    u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET)
    

# LATERAL PRESSURE STEP

# Create static step w nonlin geometry
myMod.StaticStep(name='lat_press', previous='Initial', 
    maxNumInc=1000, initialInc=0.5, minInc=1e-07, nlgeom=ON)
    

#Assign pressure load to plate surface
myMod.Pressure(name='LatPress', createStepName='lat_press', 
    region=pSurf, distributionType=UNIFORM, field='', magnitude=LatPress, 
    amplitude=UNSET)

# RIKS STEP

#Create reference point for the axial load 
#myAssembly.Set(name='Set-8', referencePoints=(
#    myAssembly.referencePoints[leftRP.id], ))

#Create riks step
myMod.StaticRiksStep(name='ult_step', previous='lat_press', 
    maxLPF=None, nodeOn=ON, 
    maximumDisplacement=-0.008, region=Region(referencePoints=(
    myAssembly.referencePoints[leftRP.id], )),
    dof=3, maxNumInc=1000, initialArcInc=0.1, minArcInc=1e-07)
    
 
#Create axial load
myMod.ConcentratedForce(cf3=-13300000, createStepName='ult_step'
    , distributionType=UNIFORM, field='', localCsys=None, name='buckle_load', 
    region=Region(referencePoints=(
    myAssembly.referencePoints[leftRP.id], )))    

# Create job and submit

mdb.Job(name='end_short', model='StiffPanel', description='', type=ANALYSIS, 
        atTime=None, waitMinutes=0, waitHours=0, queue=None, memory=90, 
        memoryUnits=PERCENTAGE, getMemoryFromAnalysis=True, 
        explicitPrecision=SINGLE, nodalOutputPrecision=SINGLE, echoPrint=OFF, 
        modelPrint=OFF, contactPrint=OFF, historyPrint=OFF, userSubroutine='', 
        scratch='', resultsFormat=ODB, multiprocessingMode=DEFAULT, numCpus=2, 
        numDomains=2, numGPUs=0)
        
mdb.jobs['end_short'].submit(consistencyChecking=OFF)
mdb.jobs['end_short'].waitForCompletion()

# POSTPROCESSING

execfile('ultstrODB.py')