%%%%%%%%%%%%%% Vector Projection Response Surface Approach %%%%%%%%%%%%%%
% 
% Created by: David Larsson,    davidlastud.ntnu.no
%
% Part of the Master Thesis:
%
% "Methods of Reliability Analysis for Marine Structures"
%
%           Norwegian University of Science and Technology
%                 Department of Marine Technology
%                            June 2015    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all
clear all
clc
format long

%% ------------------- Variable properties ---------------------------- %%

mu1=20;           % Plate thickness [mm]
mu2=100;          % Axial Force [MPa]
mu3=0.1;          % Lateral Pressure [MPa]
mu4=0;            % Elastic Modulus [MPa]
mu5=315;          % Yield Stress [MPa]


mean=[mu1 mu2 mu3 mu4 mu5];              % mean for R.V:s
dev= [0.02 0.2 0.1 0 0.07].*mean;        % std deviation of R.V
dev(4)=4;

Acs=840*6*20+5*(420*10+140*16); %cs area in mm^2

antype=1;                           %    analysis type 
                                    %    1: plate stress at midspan  
                                    %    2: ultimate collapse

f=1.5;                              % sample spread
nvar=length(mean);                  % no of R.V

%% ------------- Initial sample centered at the means ---------------- %%

guess=zeros(1,nvar); %initial guess at means;
% sample initial safety margin

cent_pt=zeros(1,nvar);   % center point in z-space
[z_samp,kim_samp]=gen_samp(cent_pt,f,mean,dev);

z_samp(1,:)=[];   %removing the first row (not used in this method)
kim_samp(1,:)=[];



%% ---------- Vector projection response surface algorithm ----------- %%

reps=0; % number of iterations
conv=1; % convergence operator
betprev=0;  % safety idnex at previous step
again=1;
while conv > 0.001;
    
reps=reps+1;            % no if iterations counter
  
ab_samp=kim_samp;       % sample for pass to ABAQUS

% transform to m and N
ab_samp(:,1)=ab_samp(:,1)./1000; %from mm to m
ab_samp(:,2)=ab_samp(:,2).*Acs;  %from MPa to N
ab_samp(:,3)=ab_samp(:,3).*1e6;  %from MPa to Pa
ab_samp(:,4)=ab_samp(:,4)./1000; %from mm to m
ab_samp(:,5)=ab_samp(:,5).*1e6;  %from MPa to Pa


%% ---------------- Performing the sampling -------------------------- %%

out = abaqus_sample(ab_samp, antype);

%% ---------------- Evaluating safety margin ------------------------- %%
if antype==1
  
    yield=ab_samp(:,5);
    %safety margin normalized wrt yield stress
    gexp=(0.8*yield-out')./(mu5*1e6); 
    
elseif antype ==2

    ax_load=samp_abaq(:,2);

    % fixed axial load at mean in the ABAQUS-script,
    % so axial capacity is LPF(out) times mu2*Acs in N.
    gexp=(0.8*out'*mu2*Acs-ax_load)./(mu2*Acs); 
    % normalize wrt mean ax load
    
end

%% -------------------- Fit hyperplane ------------------------------- %%

xmdl=fitlm(kim_samp,gexp,'linear') ;

%gathering the polynomial coefficients
pars_x=xmdl.Coefficients{:,1}; 

% partial derivatives vector
xdg=pars_x(2:end)'; 

% transforming to z-space
par_z=zeros(size(pars_x));
par_z(1)=pars_x(1)+sum(xdg.*mean);
par_z(2:end)=pars_x(2:end).*dev';

%% --------------------- FORM ---------------------------------------- %%

    beta=sqrt(sum(guess.^2));           % initial safety index

    a=par_z(1);
    b=par_z(2:end);

    diff=1;                 % convergence dummy
    betatemp=100;           % safety index dummy
    i=0;
while diff > 0.001 
        i=i+1;
        dg=b;
        alpha=-dg/sqrt(sum(dg.^2));
        beta=-a/(sum(b.*alpha));
        dir_vec(i,:)=alpha;
        saf_ind(i)=beta;
        guess=beta*alpha;
        despt(i,:)=guess;
        grad=dg;
        diff=abs(beta-betatemp);
        betatemp=beta;
end

zdes=despt(end,:);
guess=zdes;             % updating the initial guess;

%transforming the design point coord. to x-space
xdes=zdes.*dev+mean;

% partial derivatives vector
xdg=pars_x(2:end)';  

%% -------------------- Perturbing sample points --------------------- %%

delta_g=xdg';

umat=eye(nvar);
h=zeros(nvar);
for i=1:nvar
    
    u=umat(i,:)';
    h(:,i)=u-delta_g*(delta_g'*u);
    
end

% shift directions
delta=zeros(nvar);
for i=1:nvar
   for j=1:nvar
    delta(i,j)=-h(j,i)./sqrt(sum(h(:,i).^2));
   end
end

 % perturbation vectors
epsmat=0.9*ones(nvar);
for i=1:nvar
 epsmat(i,i)=1;
end


% shift magnitudes
for i=1:nvar
 eps=epsmat(i,:)';
 shift(i)=f*dev(i)*sqrt(nvar-1)*eps'*delta(i,:)';
end

% center point for next sample
kim_samp=zeros(2*nvar,nvar);
for i=1:2*nvar
kim_samp(i,:)=xdes;
end

% shifting the sampling points
k=1;            % row operator
for i=1:nvar    % i is the column operator
   kim_samp(k,i)=kim_samp(k,i)+ shift(i);
   kim_samp(k+1,i)=kim_samp(k+1,i)-shift(i);
   k=k+2;
end

%% ---------------- Check convergence and update ---------------------- %%

% store results from current iteration
safind(reps)=beta(end);
dpx(reps,:)=xdes;


% check convergence
conv=abs(safind(reps) - betprev);

% update convergence dummy
betprev=safind(reps);

end