%%%%%%%%%%%%%% Vector Projection Response Surface Approach %%%%%%%%%%%%%%
% 
% Created by: David Larsson,    davidlastud.ntnu.no
%
% Part of the Master Thesis:
%
% "Methods of Reliability Analysis for Marine Structures"
%
%           Norwegian University of Science and Technology
%                 Department of Marine Technology
%                            June 2015    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all
clear all
clc
format long

%% ------------------ Variable properties ---------------------------- %%



mu1=20;              % Plate thickness [mm]
mu2=100;             % Axial Force [MPa]
mu3=0.1;             % Lateral Pressure [MPa]
mu4=0;               % Elastic Modulus [MPa]
mu5=315;             % Yield Stress [MPa]


mu=[mu1 mu2 mu3 mu4 mu5];                  % mean for r_v:s
dev= [0.02 0.2 0.1 0 0.07].*mu;            %std deviation of basic r.v
Acs=840*6*20+5*(420*10+140*16);            %cs area in mm^2

dev(4)=4;

% parameters for weibull distributions
% corresponding to mean [100 0.1]
% and std deviation [20 0.01];
scale=[107.9975 0.1043];
shape=[5.7974 12.1535];

% Parameters for log-normal yield stress
logmu=log((mu(5)^2)/sqrt(dev(5)+mu(5)^2));
logdev=sqrt(log(dev(5)^2/(mu(5)^2)+1));


% analysis type
antype=2;                 % 1 = von-Mises stress plateside, midspan
                          % 2 = axial capacity

f=1.5;                    % sample spread factor  
nvar=length(mu);          % no of variables

%% ------- Generating initial sample centered around the means ------ %%

guess_x=mu;                 % initial guess at means;

cent_pt=zeros(1,nvar);      % center point in z-space
[z_samp,kim_samp]=gen_samp(cent_pt,f,mu,dev);

z_samp(1,:)=[];     % removing the first sample (not used in this method)
kim_samp(1,:)=[];



%% ------------------- Starting algorithm --------------------------- %%
reps=0; % number of iterations
conv=1; % convergence operator
betprev=0;  % safety idnex at previous step
again=1;
while conv > 0.001;
    
reps=reps+1;

%% --------------------- Sampling ------------------------------------ %% 

%sample for pass to ABAQUS
ab_samp=kim_samp;

% transform to mm and N
ab_samp(:,1)=ab_samp(:,1)./1000; %from mm to M
ab_samp(:,2)=ab_samp(:,2).*Acs;  %from MPa to N
ab_samp(:,3)=ab_samp(:,3).*1e6;  %from MPa to Pa
ab_samp(:,4)=ab_samp(:,4)./1000; %from mm to m
ab_samp(:,5)=ab_samp(:,5).*1e6;  %from MPa to Pa 

out = abaqus_sample(ab_samp, antype);

%% ---------------- Evaluating safety margin ------------------------- %%
if antype==1
  
    yield=ab_samp(:,5);
    %safety margin normalized wrt yield stress
    gexp=(0.8*yield-out)./(mu5*1e6); 
    
elseif antype ==2

   ax_load=ab_samp(:,2);

 % fixed axial load at mean in the ABAQUS-script,
 % so axial capacity is LPF(out) times mu2*Acs in N.
   gexp=(0.8*out*mu2*Acs-ax_load)./(mu2*Acs);  % normalize wrt mean load
                                      
    
end

%% -------------------- Fit hyperplane ------------------------------- %%
xmdl=fitlm(kim_samp,gexp,'linear') ;
par_x=xmdl.Coefficients{:,1}; %gathering the polynomial coefficients

% partial derivatives vector
xdg=par_x(2:end)';  


%% ------------------------- FORM ------------------------------------- %% 

% remove old results
clear saf_ind des_x grad

i=0;           % counter
diff=1;        % convergence operator
betatemp=100;  % beta dummy 

while diff > 0.001
i=i+1;
% approximate standard deviations
da1=dev(1);
da2=normpdf(norminv(wblcdf(guess_x(2),scale(1),shape(1))))...
    /wblpdf(guess_x(2),scale(1),shape(1));
da3=normpdf(norminv(wblcdf(guess_x(3),scale(2),shape(2))))...
    /wblpdf(guess_x(3),scale(2),shape(2));
da4=dev(4);
da5=normpdf(norminv(logncdf(guess_x(5),logmu,logdev)))...
    /lognpdf(guess_x(5),logmu,logdev);
da=[da1 da2 da3 da4 da5];

% approximate means
ma1=mu(1);
ma2=guess_x(2)-norminv(wblcdf(guess_x(2),scale(1),shape(1)))*da2;
ma3=guess_x(3)-norminv(wblcdf(guess_x(3),scale(2),shape(2)))*da3;
ma4=mu(4);
ma5=guess_x(5)-norminv(logncdf(guess_x(5),logmu,logdev))*da5;
ma=[ma1 ma2 ma3 ma4 ma5];

% polynomial coefficients
a=par_x(1);
b=par_x(2:nvar+1)';

% standardized variables
z=(guess_x-ma)./da;

% safety margin differentiated wrt standard gauss var
dg=b.*da;

% unit vectors
alpha=-dg/sqrt(sum(dg.^2));

% solve for beta
syms hlind
hsolve = solve(a + sum(b.*(hlind*alpha.*da+ma)) ==0,hlind);

beta=eval(hsolve);

% updating guess
guess_x=beta*alpha.*da+ma;

saf_ind(i)=beta;                % store safety indec
des_x(i,:)=guess_x;             % store design point
grad(i,:)=dg;                   % gradients
diff=abs(beta-betatemp);        % convergence operator
betatemp=beta;                  % dummy update
end

gradient=grad(end,:);    % storing gradient at design point
despt_x=des_x(end,:);    % storing last design point
guess_x=despt_x;         % updating the guess for next round

xdes=despt_x(end,:);     % current design point

%% --------------------- Vector Projection -------------------------- %%

delta_g=xdg';


umat=eye(nvar);
h=zeros(nvar);
for i=1:nvar
    u=umat(i,:)';
    h(:,i)=u-delta_g*(delta_g'*u);    
end

% shift directions
for i=1:nvar
   for j=1:nvar
    delta(i,j)=-h(j,i)./sqrt(sum(h(:,i).^2));
   end
end

 % perturbation vectors
epsmat=0.9*ones(nvar);
for i=1:nvar
 epsmat(i,i)=1;
end


% shift magnitudes
for i=1:nvar
 eps=epsmat(i,:)';
 shift(i)=f*dev(i)*sqrt(nvar-1)*eps'*delta(i,:)'   ;
end


kim_samp=zeros(2*nvar,nvar);
% center point for next sample
for i=1:2*nvar
kim_samp(i,:)=xdes;
end


% shifting the sampling points
k=1;            % row operator
for i=1:nvar    % i is the column operator
   kim_samp(k,i)=kim_samp(k,i)+shift(i);
   kim_samp(k+1,i)=kim_samp(k+1,i)-shift(i);
   k=k+2;
end

%% ------------------ Check convergence and update -------------------- %%

% store safety index and design point
safind(reps)=beta(end);
dpx(reps,:)=xdes;



% check convergence
conv=abs(safind(reps) - betprev);

betprev=safind(reps);

end
