%%%%%%%%%%%%%% Vector Projection Response Surface Approach %%%%%%%%%%%%%%
% 
% Created by: David Larsson,    davidla@stud.ntnu.no
%
% Part of the Master Thesis:
%
% "Methods of Reliability Analysis for Marine Structures"
%
%           Norwegian University of Science and Technology
%                 Department of Marine Technology
%                            June 2015    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all
clear all
clc
format long

%% ----------------- Variable properties ------------------------------ %%
Acs=840*6*20+5*(420*10+140*16); %cs area in mm^2

mu1=20;           % Plate thickness [mm]
mu2=100;          % Axial Force [MPa]
mu3=0.1;          % Lateral Pressure [MPa]
mu4=315;          % Yield Stress [MPa]


mean=[mu1 mu2 mu3 mu4];                  % mean for r_v:s
dev= [0.02 0.2 0.1 0.07].*mean;  %std deviation of basic r.v
Acs=840*6*20+5*(420*10+140*16); %cs area in mm^2

antype=1;               

f=1.0;                  % sample spread
nvar=length(mean);      % no of variables

%% ------------- Initial sample centered around the means ----------- %%

guess=zeros(1,nvar); %initial guess at means;
% sample initial safety margin

cent_pt=zeros(1,nvar);   % center point in z-space
[z_samp,kim_samp]=gen_samp(cent_pt,f,mean,dev);

z_samp(1,:)=[];   %removing the first row (not used in this method)
kim_samp(1,:)=[];



%% ------------ Kim-Na vector projection algorithm ------------------ %%

reps=0;     % number of iterations
conv=1;     % convergence operator
betprev=0;  % safety index at previous step
again=1;
while conv > 0.001;
    
reps=reps+1;

% evaluate sample
    for i=1:size(kim_samp,1)
       checks=dnvcheck(kim_samp(i,1),...
       kim_samp(i,2),kim_samp(i,3),kim_samp(i,4));
       smsamp(i)=0.8-checks(4);
    end

% fit hyperplane
xmdl=fitlm(kim_samp,smsamp,'linear') ;

pars_x=xmdl.Coefficients{:,1}; % gathering coefficients

% partial derivatives vector
xdg=pars_x(2:end)';  

% transforming to z-space
par_z=zeros(size(pars_x));
par_z(1)=pars_x(1)+sum(xdg.*mean);
par_z(2:end)=pars_x(2:end).*dev';


%% --------------------- FORM ---------------------------------------- %%

beta=sqrt(sum(guess.^2));           % initial safety index

a=par_z(1);
b=par_z(2:end);

diff=1;                 % convergence dummy
betatemp=100;           % safety index dummy
i=0;                    % iteration counter
while diff > 0.001 
        i=i+1;
        dg=b;
        alpha=-dg/sqrt(sum(dg.^2));
        beta=-a/(sum(b.*alpha));
        dir_vec(i,:)=alpha;
        saf_ind(i)=beta;
        guess=beta*alpha;
        despt(i,:)=guess;
        grad=dg;
        diff=abs(beta-betatemp);
        betatemp=beta;
end

zdes=despt(end,:);
guess=zdes;             % updating the initial guess;

%transforming the design point coord. to x-space
xdes=zdes.*dev+mean;

% partial derivatives vector
xdg=pars_x(2:end)';  

delta_g=xdg';


%% --------------- Perturbing the sample points ----------------------- %%

umat=eye(nvar);
h=zeros(nvar);
for i=1:nvar
    u=umat(i,:)';
    h(:,i)=u-delta_g*(delta_g'*u);
end

% shift directions
for i=1:nvar
   for j=1:nvar
    delta(i,j)=-h(j,i)./sqrt(sum(h(:,i).^2));
   end
end

% perturbation vectors
epsmat=0.9*ones(nvar);
for i=1:nvar
 epsmat(i,i)=1; 
end


% shift magnitudes
for i=1:nvar
 eps=epsmat(i,:)';
 shift(i)=f*dev(i)*sqrt(nvar-1)*eps'*delta(i,:)'   ;
end

% center point for next sample
kim_samp=zeros(2*nvar,nvar);
for i=1:2*nvar
kim_samp(i,:)=xdes;
end


% shifting the sampling points
k=1;            % row operator
for i=1:nvar    % i is the column operator
   kim_samp(k,i)=kim_samp(k,i)+shift(i);
   kim_samp(k+1,i)=kim_samp(k+1,i)-shift(i);
   k=k+2;
end

%% -------------- Convergence check and update ------------------------ %%

safind(reps)=beta(end);
dpx(reps,:)=xdes;

conv=abs(safind(reps) - betprev);

betprev=safind(reps);

end
