%%%%%%%%%%%%%% Vector Projection Response Surface Approach %%%%%%%%%%%%%%
% 
% Created by: David Larsson,    davidlastud.ntnu.no
%
% Part of the Master Thesis:
%
% "Methods of Reliability Analysis for Marine Structures"
%
%           Norwegian University of Science and Technology
%                 Department of Marine Technology
%                            June 2015    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


close all
clear all
clc
format long

%% -------------------- Variable properties --------------------------- %%

mu1=20;         % Plate thickness [mm]
mu2=100;          % Axial Force [MPa]
mu3=0.1;          % Lateral Pressure [MPa]
mu4=315;          % Yield Stress [MPa]

mu=[mu1 mu2 mu3 mu4];                    % mean for r_v:s
dev= [0.02 0.2 0.1 0.07].*mu;            % std deviation of basic r.v

Acs=840*6*20+5*(420*10+140*16);          % cs area in mm^2

% parameters for weibull distributions
% corresponding to mean [100 0.1]
% std deviation [20 0.01];
scale=[107.9975 0.1043];
shape=[5.7974 12.1535];

% Parameters for log-normal yield stress
logmu=log((mu(4)^2)/sqrt(dev(4)+mu(4)^2));
logdev=sqrt(log(dev(4)^2/(mu(4)^2)+1));

f=1.0;              % sample spread
nvar=length(mu);    % no of variables

%% ------------------- Initial sample -------------------------------- %%

guess_x=mu; %initial guess at means;

cent_pt=zeros(1,nvar);   % center point in z-space
[z_samp,kim_samp]=gen_samp(cent_pt,f,mu,dev);

% removing the first row (not used in this method)
z_samp(1,:)=[];   
kim_samp(1,:)=[];



%% ---------------- Vector Projection algorithm ---------------------- %%

reps=0;     % number of iterations
conv=1;     % convergence operator
betprev=0;  % safety index at previous step
while conv > 0.001;
    
reps=reps+1;

%% ----------------- Evaluate safety margin -------------------------- %%

for i=1:size(kim_samp,1)
  checks=dnvcheck(kim_samp(i,1),kim_samp(i,2),kim_samp(i,3),kim_samp(i,4));
  smsamp(i)=0.8-checks(4);
end

% fit hyperplane
xmdl=fitlm(kim_samp,smsamp,'linear') ;

par_x=xmdl.Coefficients{:,1}; % gathering the polynomial coefficients

% partial derivatives vector
xdg=par_x(2:end)';  

%% ------------------------ FORM -------------------------------------- %%

% remove old results
clear saf_ind des_x grad

i=0;           % counter
diff=1;        % convergence operator
betatemp=100;  % beta dummy 

while diff > 0.001
i=i+1;
% approximate standard deviations
da1=dev(1);
da2=normpdf(norminv(wblcdf(guess_x(2),scale(1),shape(1))))...
    /wblpdf(guess_x(2),scale(1),shape(1));
da3=normpdf(norminv(wblcdf(guess_x(3),scale(2),shape(2))))...
    /wblpdf(guess_x(3),scale(2),shape(2));
%da4=dev(4);
da4=normpdf(norminv(logncdf(guess_x(4),logmu,logdev)))...
    /lognpdf(guess_x(4),logmu,logdev);
da=[da1 da2 da3 da4];

% approximate means
ma1=mu(1);
ma2=guess_x(2)-norminv(wblcdf(guess_x(2),scale(1),shape(1)))*da2;
ma3=guess_x(3)-norminv(wblcdf(guess_x(3),scale(2),shape(2)))*da3;
ma4=guess_x(4)-norminv(logncdf(guess_x(4),logmu,logdev))*da4;
ma=[ma1 ma2 ma3 ma4];


a=par_x(1);
b=par_x(2:nvar+1)';

% standardized variables
z=(guess_x-ma)./da;

% safety margin differentiated wrt standard gauss var
dg=b.*da;

% unit vectors
alpha=-dg/sqrt(sum(dg.^2));

syms hlind
hsolve = solve(a + sum(b.*(hlind*alpha.*da+ma)) ==0,hlind);

beta=eval(hsolve)

% updating guess
guess_x=beta*alpha.*da+ma;

% store data
saf_ind(i)=beta;
des_x(i,:)=guess_x;
grad(i,:)=dg;
diff=abs(beta-betatemp);
betatemp=beta;
end

gradient=grad(end,:);  % storing gradient at design point
despt_x=des_x(end,:);  % storing last design point
guess_x=despt_x;        % updating the guess for next round

xdes=despt_x(end,:);




delta_g=xdg';

%% --------------------- Perturb sampling points ---------------------- %%

umat=eye(nvar);
h=zeros(nvar);

for i=1:nvar
    u=umat(i,:)';
    h(:,i)=u-delta_g*(delta_g'*u);
end

% shift directions
for i=1:nvar
   for j=1:nvar
    delta(i,j)=-h(j,i)./sqrt(sum(h(:,i).^2));
   end
end

% perturbation vectors
epsmat=0.9*ones(nvar);
for i=1:nvar
 epsmat(i,i)=1;
end


% perturbation magnitudes
for i=1:nvar
 eps=epsmat(i,:)';
 shift(i)=f*dev(i)*sqrt(nvar-1)*eps'*delta(i,:)'   ;
end

kim_samp=zeros(2*nvar,nvar);
% center point for next sample
for i=1:2*nvar
kim_samp(i,:)=xdes;
end


% shifting the sampling points
k=1;            % row operator
for i=1:nvar    % i is the column operator
   kim_samp(k,i)=kim_samp(k,i)+shift(i);
   kim_samp(k+1,i)=kim_samp(k+1,i)-shift(i);
   k=k+2;
end


% store results
safind(reps)=beta(end);
dpx(reps,:)=xdes;

% update convergence and dummy
conv=abs(safind(reps) - betprev);
betprev=safind(reps);

end
